@@ -9,7 +9,8 @@ from ckan.lib.navl.dictization_functions import DataError
 from ckan.lib.redis import connect_to_redis
 from ckan.logic import side_effect_free
 from ckan.plugins import toolkit
-from ckan.types import Context, DataDict
+
+# from ckan.types import Context, DataDict
 from requests import get as get_url
 from sqlalchemy.exc import InternalError as SQLAlchemyError
 
@@ -20,8 +21,8 @@ log = logging.getLogger(__name__)
 
 
 def request_reset_key(
-    context: Context,
-    data_dict: DataDict,
+    context,  #: Context,
+    data_dict,  #: DataDict,
 ):
     """Request a reset key (login token) to be sent by email.
 
@@ -58,14 +59,16 @@ def request_reset_key(
 
     if not user:
         # A user with this email address doesn't yet exist in CKAN, so create one.
-        user = _create_user(email)
+        new_user_email = _create_user(email)
         log.debug(f"Created user {str(email)}")
+        user = util.get_user_from_email(new_user_email)
+        send_welcome_email(user)
 
     if user:
         # make sure is not deleted
         if user.state == "deleted":
             raise toolkit.ValidationError(
-                {"user": f"User with email {email} was deleted"}
+                {"user": f"User with email {email} was deleted already. Contact Admin."}
             )
         try:
             send_user_reset_key(user)
@@ -85,7 +88,10 @@ def request_reset_key(
 
 
 def _create_user(email):
-    """Create a new user and send welcome email."""
+    """Create a new user and send welcome email.
+
+    Returns the user email.
+    """
     # first check temporary quota
     util.check_new_user_quota()
 
@@ -96,10 +102,9 @@ def _create_user(email):
             "name": util.get_new_username(email),
             "password": util.generate_password(),
         }
-        user = toolkit.get_action("user_create")(
+        user_dict = toolkit.get_action("user_create")(
             context={"ignore_auth": True}, data_dict=data_dict
         )
-        send_welcome_email(user)
     except SQLAlchemyError as error:
         exception_message = f"{error}"
         log.error(f"Failed to create user: {error}")
@@ -109,13 +114,17 @@ def _create_user(email):
             )
         else:
             raise DataError from error("Internal error creating a new user")
+    except Exception as e:
+        log.error(e)
+        log.error("Error creating new user")
+        raise Exception from e
 
-    return user
+    return user_dict.get("email")
 
 
 def request_api_token(
-    context: Context,
-    data_dict: DataDict,
+    context,  #: Context,
+    data_dict,  #: DataDict,
 ):
     """Get API token for a user, if reset key valid.
 
@@ -177,8 +186,8 @@ def request_api_token(
 
 
 def request_api_token_azure_ad(
-    context: Context,
-    data_dict: DataDict,
+    context,  #: Context,
+    data_dict,  #: DataDict,
 ):
     """Get API token for a user, if valid Azure AD token provided.
 
@@ -247,8 +256,8 @@ def request_api_token_azure_ad(
 
 @side_effect_free
 def revoke_api_token_no_auth(
-    context: Context,
-    data_dict: DataDict,
+    context,  #: Context,
+    data_dict,  #: DataDict,
 ):
     """Revoke API token for a user, without requiring auth.
 
@@ -307,8 +316,8 @@ def revoke_api_token_no_auth(
 
 @side_effect_free
 def get_current_user_and_renew_api_token(
-    context: Context,
-    data_dict: DataDict,
+    context,  #: Context,
+    data_dict,  #: DataDict,
 ):
     """Return CKAN user and renew API token.
 
@@ -343,7 +352,7 @@ def get_current_user_and_renew_api_token(
         log.warning(f"Could not find a user for ID: {user_id}")
         return {"message": f"could not find a user for id: {user_id}"}
 
-    if user:
+    if _check_token_valid(context, data_dict):
         expiry = config.get("expire_api_token.default_lifetime", 3)
         units = config.get("expire_api_token.default_unit", 86400)
         token_json = util.renew_main_token(user_id, expiry, units)
@@ -353,3 +362,63 @@ def get_current_user_and_renew_api_token(
         }
 
     return {"message": "failed"}
+
+
+@side_effect_free
+def check_token_valid(
+    context,  #: Context,
+    data_dict,  #: DataDict,
+):
+    """Check if the token is valid and user is authenticated.
+
+    Allows for verifying user auth without renewing token.
+    Useful for verifying a token from microservices.
+
+    Returns:
+        bool: True if valid, False if not.
+    """
+    if _check_token_valid(context, data_dict):
+        return True
+
+    return False
+
+
+def _check_token_valid(
+    context,  #: Context,
+    data_dict,  #: DataDict,
+):
+    """Verify the JWT API token.
+
+    Returns:
+        bool: True if valid, False if not.
+    """
+    log.debug("start function _check_token_valid")
+
+    if (user_id := context.get("user", "")) != "":
+        log.debug("User ID extracted from context user key")
+    elif user_obj := context.get("auth_user_obj", None):
+        log.debug("User ID extracted from context auth_user_obj key")
+        user_id = user_obj.id
+    else:
+        return {
+            "message": "API token is invalid or missing from Authorization header",
+        }
+
+    user = None
+    try:
+        log.info(f"Getting user details with user_id: {user_id}")
+        user = toolkit.get_action("user_show")(
+            data_dict={
+                "id": user_id,
+            },
+        )
+
+    except Exception as e:
+        log.error(str(e))
+        log.warning(f"Could not find a user for ID: {user_id}")
+        return {"message": f"could not find a user for id: {user_id}"}
+
+    if user:
+        return True
+
+    return False
