@@ -6,6 +6,7 @@ from json import loads as load_json
 from ckan.plugins import SingletonPlugin, implements, interfaces, toolkit
 
 from ckanext.passwordless_api.logic import (
+    check_token_valid,
     get_current_user_and_renew_api_token,
     request_api_token,
     request_api_token_azure_ad,
@@ -68,9 +69,10 @@ class PasswordlessAPIPlugin(SingletonPlugin):
             "passwordless_request_api_token_azure_ad": request_api_token_azure_ad,
             "passwordless_revoke_api_token": revoke_api_token_no_auth,
             "passwordless_get_user": get_current_user_and_renew_api_token,
+            "passwordless_introspect": check_token_valid,
         }
 
-    # IActions
+    # IMiddleware
     def make_middleware(self, app, config):
         """Create middleware for the Flask app."""
 
@@ -81,11 +83,15 @@ class PasswordlessAPIPlugin(SingletonPlugin):
                 return response
 
             try:
-                # token present in both renew_api_token and get_user
-                token = load_json(response.data).get("result").get("token")
-                if not token:
+                # token already present in both renew_api_token and get_user
+                # simply return response with token
+                # also set direct_passthrough = False to avoid warnings
+                response.direct_passthrough = False
+                if not (token := load_json(response.data).get("result").get("token")):
                     return response
             except Exception:
+                # Required to bypass errors and continue loading
+                # Do not pollute logs
                 return response
 
             log.debug(
